#!/bin/bash -x
if [ "${1}" != "" ] ; then
  src_name="${1}"
else
  src_name="StarForce"
fi

src_rom="${src_name}.rom"
dst_rom="${src_name}-ex.rom"
tmp_rom="${src_name}-ex-tmp.rom"
name="${src_name}-"

asms="
StarForce-07a0c.asm
StarForce-07c00.asm
StarForce-08000.asm

StarForce-02a28.asm
StarForce-00014.asm

StarForce-05d74.asm
StarForce-064b4.asm

StarForce-010cf.asm
StarForce-023d3.asm
StarForce-02404.asm
StarForce-025bf.asm

StarForce-02800.asm
StarForce-03077.asm
"

asms2="
"

if ! [ -d "obj" ] ; then
  mkdir -p obj
fi

if ! [ -f "${dst_rom}" ] ; then
  cp -v "${src_rom}" "${tmp_rom}"
else
  cp -v "${dst_rom}" "${tmp_rom}"
fi


for af in ${asms} ; do
  bf="`basename ${af} .asm`.bin"
  sb="`basename ${af} .asm`.symbol"
  pb="`basename ${af} .asm`.publics"
  echo "##" "${af}" "${bf}" "${sb}" "##"

  ninf=""
  for inf in `awk '/include[ \t].+/{gsub(/[\\\\]{2}/,"/"); gsub(/\"/,""); print $2}' "${af}"` ; do
    if [ "${inf}" -nt "obj/${bf}" ] ; then
      echo "# ${inf} newer than ${bf}"
      ninf=1;
    fi
  done

  if [ "${af}" -nt "obj/${bf}" -o "" != "${ninf}" ] ; then
    echo "# assembling ${af}"
    /c/app/pasmo-0.5.3/pasmo.exe --alocal "${af}" "obj/${bf}" "obj/${sb}" "obj/${pb}"
    echo `hexdump -ve '1 1 "%02X\n"' "obj/${bf}" | awk '{l[FNR-1]=$0}END{for(i=0;i<FNR;i++)print l[FNR-i-1]}' | sed -re "/[^0]+/Q" | wc -l` bytes free
  fi

  if [ "obj/${bf}" -nt "${dst_rom}" ] ; then
    echo "# byte copy from obj/${bf}"
    s=`echo "${bf}" | sed -nre "s/${name}([^-]+-)*([0-9a-fA-F]+).bin$/0x\\2/p"`
    s=`printf "%d" "${s}"`
    l=`wc -c "obj/${bf}" | awk '{print $1}'`
    echo "${af}" "${bf}" "${s}" "${l}"
    cat obj/$bf | dd of="${tmp_rom}" bs=1 count="${l}" seek="${s}" conv=notrunc
  fi

  if [ "obj/${sb}" -nt "${dst_rom}" ] ; then
	echo "# bytes replace from obj/${sb}"
    a=(`cat "obj/${sb}" | sed -nre "s/val_[^_]{2}_in_rom_([^ ]{4,5}) EQU 000([^H]{2})H/\\1 \\2/p"`)
    while [ "${#a[@]}" -ge "2" ] ; do
      s=$((16#${a[0]}))
      echo "${sb}" "${a[0]}" "${a[1]}" "${s}"
      printf "\\x${a[1]}" | dd of="${tmp_rom}" bs=1 count=1 seek="${s}" conv=notrunc
      unset a[0]
      unset a[1]
      a=("${a[@]}")
    done

	echo "# words replace from obj/${sb}"
    a=(`cat "obj/${sb}" | sed -nre "s/val_[^_]{4}_in_rom_([^ ]{4,5}) EQU 0([^H]{2})([^H]{2})H/\\1 \\2 \\3/p"`)
    while [ "${#a[@]}" -ge "3" ] ; do
      s=$((16#${a[0]}))
      echo "${sb}" "${a[0]}" "${a[1]}" "${a[2]}" "${s}"
      printf "\\x${a[2]}\\x${a[1]}" | dd of="${tmp_rom}" bs=1 count=2 seek="${s}" conv=notrunc
      unset a[0]
      unset a[1]
      unset a[2]
      a=("${a[@]}")
    done
  fi
done
mv -v "${tmp_rom}" "${dst_rom}"
